@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2017 The Cacti Group                                 |
+ | Copyright (C) 2004-2018 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,7 +13,7 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | Cacti: The Complete RRDTool-based Graphing Solution                     |
+ | Cacti: The Complete RRDtool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
  | This code is designed, written, and maintained by the Cacti Group. See  |
  | about.php and/or the AUTHORS file for specific developer information.   |
@@ -47,7 +47,7 @@ if (read_config_option('auth_method') == '2') {
 		/* No user - Bad juju! */
 		$username = '';
 		cacti_log('ERROR: No username passed with Web Basic Authentication enabled.', false, 'AUTH');
-		auth_display_custom_error_message( __('Web Basic Authentication configured, but no username was passed from the web server. Please make sure you have authentication enabled on the web server.') );
+		auth_display_custom_error_message(__('Web Basic Authentication configured, but no username was passed from the web server. Please make sure you have authentication enabled on the web server.'));
 		exit;
 	}
 } else {
@@ -63,6 +63,7 @@ $username = sanitize_search_string($username);
 $version  = get_cacti_version();
 
 /* process login */
+$user         = array();
 $copy_user    = false;
 $user_auth    = false;
 $user_enabled = 1;
@@ -71,37 +72,53 @@ $ldap_error_message = '';
 $realm        = 0;
 
 if (get_nfilter_request_var('action') == 'login') {
-	if (get_nfilter_request_var('realm') == '1') {
+	$frv_realm = get_nfilter_request_var('realm');
+	if ($frv_realm == '1') {
 		$auth_method = 1;
 	} else {
 		$auth_method = read_config_option('auth_method');
 	}
 
+	cacti_log("DEBUG: User '" . $username . "' attempting to login with realm ". $frv_realm . ", using method " . $auth_method, false, 'AUTH', POLLER_VERBOSITY_DEBUG);
+
+	// Realms of 1 or below are internal
+	$auth_local_required = ($frv_realm < 2);
+
 	switch ($auth_method) {
 	case '0':
 		/* No auth, no action, also shouldn't get here */
+		$auth_local_required = false;
 		exit;
 
 		break;
 	case '2':
 		/* Web Basic Auth */
+		$auth_local_required = false;
 		$copy_user = true;
 		$user_auth = true;
 		$realm = 2;
+
 		/* Locate user in database */
-		$user = db_fetch_row_prepared('SELECT * FROM user_auth WHERE username = ? AND realm = 2', array($username));
+		$user = db_fetch_row_prepared('SELECT *
+			FROM user_auth
+			WHERE username = ?
+			AND realm = 2',
+			array($username));
 
 		if (!$user && read_config_option('user_template') == '0' && read_config_option('guest_user') == '0') {
 			cacti_log("ERROR: User '" . $username . "' authenticated by Web Server, but both Template and Guest Users are not defined in Cacti.  Exiting.", false, 'AUTH');
+
 			$username = htmlspecialchars($username);
-			auth_display_custom_error_message( __('%s authenticated by Web Server, but both Template and Guest Users are not defined in Cacti.', $username) );
+
+			display_custom_error_message(__('%s authenticated by Web Server, but both Template and Guest Users are not defined in Cacti.', $username));
+			header('Location: index.php?header=false');
 			exit;
 		}
 
 		break;
 	case '3':
 		/* LDAP Auth */
- 		if (get_nfilter_request_var('realm') == '2' && get_nfilter_request_var('login_password') != '') {
+ 		if ($frv_realm == '2' && get_nfilter_request_var('login_password') != '') {
 			/* include LDAP lib */
 			include_once('./lib/ldap.php');
 
@@ -115,7 +132,6 @@ if (get_nfilter_request_var('action') == 'login') {
 				$ldap_error = true;
 				$ldap_error_message =  __('LDAP Search Error: %s', $ldap_dn_search_response['error_text']);
 				$user_auth = false;
-				$user = array();
 			}
 
 			if (!$ldap_error) {
@@ -142,7 +158,6 @@ if (get_nfilter_request_var('action') == 'login') {
 					$ldap_error = true;
 					$ldap_error_message = __('LDAP Error: %s', $ldap_auth_response['error_text']);
 					$user_auth = false;
-					$user = array();
 				}
 			}
 
@@ -150,55 +165,50 @@ if (get_nfilter_request_var('action') == 'login') {
 
 		break;
 	case '4':
-		if (get_request_var('realm') > 0) {
-			domains_login_process();
+		cacti_log("DEBUG: User '" . $username . "' attempting domain lookup for realm " . $frv_realm . " with " . ($auth_local_required ? '':'no') . " local lookup", false, 'AUTH', POLLER_VERBOSITY_DEBUG);
 
-			break;
+		if ($frv_realm > 0) {
+			domains_login_process($username);
 		}
 
-		/* continue on to normal login process */
+		break;
 	default:
-		secpass_login_process();
+		$auth_local_required = true;
+
+		break;
+	}
+
+	cacti_log("DEBUG: User '" . $username . "' attempt login locally? " . ($auth_local_required ? 'Yes':'No'), false, 'AUTH', POLLER_VERBOSITY_DEBUG);
+	if ($auth_local_required) {
+		secpass_login_process($username);
 
 		if (!api_plugin_hook_function('login_process', false)) {
 			/* Builtin Auth */
-			$user = array();
-			if ((!$user_auth) && (!$ldap_error)) {
+			if ((!$user_auth)) {
 				$stored_pass = db_fetch_cell_prepared('SELECT password
 					FROM user_auth
 					WHERE username = ? AND realm = 0',
 					array($username));
 
 				if ($stored_pass != '') {
-					if (function_exists('password_verify')) {
-						$p = get_nfilter_request_var('login_password');
-
-						if (password_verify($p, $stored_pass)) {
-							$user = db_fetch_row_prepared('SELECT *
-								FROM user_auth
-								WHERE username = ? AND realm = 0',
-								array($username));
-
-							if (password_needs_rehash($p, PASSWORD_DEFAULT)) {
-								$p = password_hash($p, PASSWORD_DEFAULT);
-								db_execute_prepared('UPDATE user_auth
-									SET password = ?
-									WHERE username = ?',
-									array($p, $username));
-							}
-						}
-					}
-
-					if (!sizeof($user)) {
-						$p = md5(get_nfilter_request_var('login_password'));
+					$p = get_nfilter_request_var('login_password');
+					$valid = compat_password_verify($p, $stored_pass);
 
+					cacti_log("DEBUG: User '" . $username . "' password is " . ($valid?'':'in') . "valid", false, 'AUTH', POLLER_VERBOSITY_DEBUG);
+					if ($valid) {
 						$user = db_fetch_row_prepared('SELECT *
 							FROM user_auth
-							WHERE username = ? AND password = ? AND realm = 0',
-							array($username, $p));
+							WHERE username = ? AND realm = 0',
+							array($username));
+
+						if (compat_password_needs_rehash($stored_pass, PASSWORD_DEFAULT)) {
+							$p = compat_password_hash($p, PASSWORD_DEFAULT);
+							db_execute_prepared('UPDATE user_auth
+								SET password = ?
+								WHERE username = ?',
+								array($p, $username));
+						}
 					}
-				} else {
-					$user = array();
 				}
 			}
 		}
@@ -208,16 +218,19 @@ if (get_nfilter_request_var('action') == 'login') {
 	/* Create user from template if requested */
 	if (!sizeof($user) && $copy_user && read_config_option('user_template') != '0' && $username != '') {
 		cacti_log("WARN: User '" . $username . "' does not exist, copying template user", false, 'AUTH');
+
 		/* check that template user exists */
 		if (db_fetch_row_prepared('SELECT id FROM user_auth WHERE username = ? AND realm = 0', array(read_config_option('user_template')))) {
 			/* template user found */
 			user_copy(read_config_option('user_template'), $username, 0, $realm);
+
 			/* requery newly created user */
 			$user = db_fetch_row_prepared('SELECT * FROM user_auth WHERE username = ? AND realm = ?', array($username, $realm));
 		} else {
 			/* error */
+			display_custom_error_message(__('Template user %s does not exist.', read_config_option('user_template')));
 			cacti_log("LOGIN: Template user '" . read_config_option('user_template') . "' does not exist.", false, 'AUTH');
-			auth_display_custom_error_message( __('Template user %s does not exist.', read_config_option('user_template')) );
+			header('Location: index.php?header=false');
 			exit;
 		}
 	}
@@ -226,14 +239,20 @@ if (get_nfilter_request_var('action') == 'login') {
 	$guest_user = false;
 	if ((!sizeof($user)) && ($user_auth) && (read_config_option('guest_user') != '0')) {
 		/* Locate guest user record */
-		$user = db_fetch_row_prepared('SELECT * FROM user_auth WHERE username = ?', array(read_config_option('guest_user')));
+		$user = db_fetch_row_prepared('SELECT *
+			FROM user_auth
+			WHERE username = ?',
+			array(read_config_option('guest_user')));
+
 		if ($user) {
 			cacti_log("LOGIN: Authenticated user '" . $username . "' using guest account '" . $user['username'] . "'", false, 'AUTH');
+
 			$guest_user = true;
 		} else {
 			/* error */
-			auth_display_custom_error_message( __('Guest user %s does not exist.', read_config_option('guest_user')) );
+			display_custom_error_message(__('Guest user %s does not exist.', read_config_option('guest_user')));
 			cacti_log("LOGIN: Unable to locate guest user '" . read_config_option('guest_user') . "'", false, 'AUTH');
+			header('Location: index.php?header=false');
 			exit;
 		}
 	}
@@ -267,7 +286,8 @@ if (get_nfilter_request_var('action') == 'login') {
 		$user_enabled = $user['enabled'];
 		if ($user_enabled != 'on') {
 			/* Display error */
-			auth_display_custom_error_message( __('Access Denied, user account disabled.') );
+			display_custom_error_message(__('Access Denied, user account disabled.'));
+			header('Location: index.php?header=false');
 			exit;
 		}
 
@@ -310,10 +330,10 @@ if (get_nfilter_request_var('action') == 'login') {
 				/* because we use plugins, we can't redirect back to graph_view.php if they don't
 				 * have console access
 				 */
-				if (isset($_SERVER["REDIRECT_URL"])) {
-					$referer = $_SERVER["REDIRECT_URL"];
-					if (isset($_SERVER["REDIRECT_QUERY_STRING"])) {
-						$referer .= '?' . $_SERVER["REDIRECT_QUERY_STRING"] . ($newtheme ? '&newtheme=1':'');
+				if (isset($_SERVER['REDIRECT_URL'])) {
+					$referer = $_SERVER['REDIRECT_URL'];
+					if (isset($_SERVER['REDIRECT_QUERY_STRING'])) {
+						$referer .= '?' . $_SERVER['REDIRECT_QUERY_STRING'] . ($newtheme ? '&newtheme=1':'');
 					}
 				} else if (isset($_SERVER['HTTP_REFERER'])) {
 					$referer = $_SERVER['HTTP_REFERER'];
@@ -334,7 +354,11 @@ if (get_nfilter_request_var('action') == 'login') {
 				} elseif (sizeof(db_fetch_assoc_prepared('SELECT realm_id FROM user_auth_realm WHERE realm_id = 8 AND user_id = ?', array($_SESSION['sess_user_id']))) == 0) {
 					header('Location: graph_view.php' . ($newtheme ? '?newtheme=1':''));
 				} else {
-					header("Location: $referer" . ($newtheme ? '?newtheme=1':''));
+					$param_char = '?';
+					if (substr_count($referer, '?')) {
+						$param_char = '&';
+					}
+					header('Location: ' . $referer . ($newtheme ? $param_char . 'newtheme=1':''));
 				}
 
 				break;
@@ -353,8 +377,9 @@ if (get_nfilter_request_var('action') == 'login') {
 	} else {
 		if ((!$guest_user) && ($user_auth)) {
 			/* No guest account defined */
-			auth_display_custom_error_message( __('Access Denied, please contact you Cacti Administrator.') );
+			display_custom_error_message(__('Access Denied, please contact you Cacti Administrator.'));
 			cacti_log('LOGIN: Access Denied, No guest enabled or template user to copy', false, 'AUTH');
+			header('Location: index.php');
 			exit;
 		} else {
 			/* BAD username/password builtin and LDAP */
@@ -364,8 +389,9 @@ if (get_nfilter_request_var('action') == 'login') {
 }
 
 /* auth_display_custom_error_message - displays a custom error message to the browser that looks like
-     the pre-defined error messages
-   @arg $message - the actual text of the error message to display */
+   the pre-defined error messages
+   @arg $message - the actual text of the error message to display
+*/
 function auth_display_custom_error_message($message) {
 	global $config;
 
@@ -374,17 +400,12 @@ function auth_display_custom_error_message($message) {
 
 	/* print error */
 	print '<!DOCTYPE html>';
-	print "<html>\n<head>\n";
-	print '     <title>' . 'Cacti' . "</title>\n";
-	print "     <meta http-equiv='Content-Type' content='text/html;charset=utf-8'>";
-	print "		<meta content='width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0' name='viewport'>\n";
-	print "		<meta name='apple-mobile-web-app-capable' content='yes'>\n";
-	print "		<meta name='mobile-web-app-capable' content='yes'>\n";
-	print "     <meta http-equiv='X-UA-Compatible' content='IE=Edge,chrome=1'>";
-	print "     <link href=\"include/main.css\" type=\"text/css\" rel=\"stylesheet\">";
+	print "<html>\n";
+	print "<head>\n";
+	html_common_header(__('Cacti'));
 	print "</head>\n";
 	print "<body>\n<br><br>\n";
-	display_custom_error_message($message);
+	print $message . "\n";
 	print "</body>\n</html>\n";
 }
 
@@ -405,7 +426,6 @@ function domains_login_process() {
 			$ldap_error = true;
 			$ldap_error_message = __('LDAP Search Error: %s', $ldap_dn_search_response['error_text']);
 			$user_auth = false;
-			$user = array();
 		}
 
 		if (!$ldap_error) {
@@ -434,8 +454,9 @@ function domains_login_process() {
 						$user = db_fetch_row_prepared('SELECT * FROM user_auth WHERE username = ? AND realm = ?', array($username, $realm));
 					} else {
 						/* error */
+						display_custom_error_message(__('Template user %s does not exist.', $template_username));
 						cacti_log("LOGIN: Template user '" . $template_username . "' does not exist.", false, 'AUTH');
-						auth_display_custom_error_message( __('Template user %s does not exist.', $template_username) );
+						header('Location: index.php?header=false');
 						exit;
 					}
 				}
@@ -445,7 +466,6 @@ function domains_login_process() {
 				$ldap_error = true;
 				$ldap_error_message = __('LDAP Error: %s', $ldap_auth_response['error_text']);
 				$user_auth = false;
-				$user = array();
 			}
 		}
 
@@ -538,29 +558,7 @@ $selectedTheme = get_selected_theme();
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
 <html>
 <head>
-	<title><?php print api_plugin_hook_function('login_title', __('Login to Cacti'));?></title>
-	<meta http-equiv='Content-Type' content='text/html;charset=utf-8'>
-	<meta http-equiv='X-UA-Compatible' content='IE=Edge,chrome=1'>
-	<meta content='width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0' name='viewport'>
-	<meta name='apple-mobile-web-app-capable' content='yes'>
-	<meta name='mobile-web-app-capable' content='yes'>
-	<meta name='robots' content='noindex,nofollow'>
-	<link href='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/jquery-ui.css' type='text/css' rel='stylesheet'>
-	<link href='<?php echo $config['url_path']; ?>include/fa/css/font-awesome.css' type='text/css' rel='stylesheet'>
-	<link href='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/main.css' type='text/css' rel='stylesheet'>
-	<link href='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/images/favicon.ico' rel='shortcut icon'>
-    <link href='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/images/cacti_logo.gif' rel='icon' sizes='96x96'>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery.js' language='javascript'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery-migrate.js' language='javascript'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery-ui.js' language='javascript'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/js.storage.js'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery.cookie.js' language='javascript'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery.hotkeys.js'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery.metadata.js'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/js/jquery.tablesorter.js'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/layout.js'></script>
-	<script type='text/javascript' src='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/main.js'></script>
-	<script type='text/javascript'>var theme='<?php print $selectedTheme;?>';</script>
+	<?php html_common_header(api_plugin_hook_function('login_title', __('Login to Cacti')));?>
 	<script type='text/javascript'>
 	$(function() {
 			$('#login_username').focus();
@@ -584,7 +582,7 @@ $selectedTheme = get_selected_theme();
 						'action' => get_nfilter_request_var('action')));
 				?>
 				<div class='loginTitle'>
-					<p><? print __('Enter your Username and Password below');?></p>
+					<p><?php print __('Enter your Username and Password below');?></p>
 				</div>
 				<div class='cactiLogin'>
 					<table class='cactiLoginTable'>
@@ -630,9 +628,9 @@ $selectedTheme = get_selected_theme();
 							<td>
 								<select id='realm' name='realm'><?php
 									if (sizeof($realms)) {
-									foreach($realms as $index => $realm) {
-										print "\t\t\t\t\t<option value='" . $index . "'" . ($realm['selected'] ? ' selected':'') . '>' . htmlspecialchars($realm['name']) . "</option>\n";
-									}
+										foreach($realms as $index => $realm) {
+											print "\t\t\t\t\t<option value='" . $index . "'" . ($realm['selected'] ? ' selected':'') . '>' . htmlspecialchars($realm['name']) . "</option>\n";
+										}
 									}
 									?>
 								</select>
