@@ -416,7 +416,11 @@ function form_actions() {
 					$form_array += array($field_name => $fields_host_edit[$field_name]);
 
 					$form_array[$field_name]['value'] = '';
-					$form_array[$field_name]['description'] = '';
+
+					if (read_config_option('hide_form_description') == 'on') {
+						$form_array[$field_name]['description'] = '';
+					}
+
 					$form_array[$field_name]['form_id'] = 0;
 					$form_array[$field_name]['sub_checkbox'] = array(
 						'name' => 't_' . $field_name,
@@ -435,6 +439,8 @@ function form_actions() {
 
 			device_javascript();
 
+			device_change_javascript();
+
 			$save_html = "<input type='button' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __esc('Continue') . "' title='" . __esc('Change Device(s) SNMP Options') . "'>";
 		} elseif (get_request_var('drp_action') == '5') { // Clear Statisitics for Selected Devices
 			print "<tr>
@@ -697,7 +703,7 @@ function host_edit() {
 				<div class='cactiTableButton'>
 					<span>
 						<a class='linkCopyDark cactiTableCopy' id='copyToClipboard<?php print $dbg_copy_uid;?>'><?php print __('Copy');?></a>
-						<a id='dbghide' class='deletequery fa fa-remove' href='#'><?php print __('Hide');?></a>
+						<a id='dbghide' class='fa fa-remove' href='#'><?php print __('Hide');?></a>
 					</span>
 				</div>
 			</div>
@@ -963,6 +969,48 @@ function device_reindex_methods($item, $host) {
 	}
 }
 
+function device_change_javascript() {
+	?>
+	<script type="text/javascript">
+	function disableField(id) {
+		$('#'+id).prop('disabled', true);
+
+		if ($('#'+id).button('instance')) {
+			$('#'+id).button('disable');
+		} else if ($('#'+id).selectmenu('instance')) {
+			$('#'+id).selectmenu('disable');
+		}
+	}
+
+	function enableField(id) {
+		$('#'+id).prop('disabled', false);
+
+		if ($('#'+id).button('instance')) {
+			$('#'+id).button('enable');
+		} else if ($('#'+id).selectmenu('instance')) {
+			$('#'+id).selectmenu('enable');
+		}
+	}
+
+	$(function() {
+		$('input[id^="t_"]').click(function() {
+			id = $(this).attr('id').substring(2);
+			if ($(this).is(':checked')) {
+				enableField(id);
+			} else {
+				disableField(id);
+			}
+		});
+
+		$('input[id^="t_"]').each(function() {
+			id = $(this).attr('id').substring(2);
+			disableField(id);
+		});
+	});
+	</script>
+	<?php
+}
+
 function device_javascript() {
 	?>
 	<script type="text/javascript">
@@ -996,36 +1044,36 @@ function device_javascript() {
 
 		switch(availability_method) {
 		case '0': // none
-			$('#row_ping_method').css('display', 'none');
-			$('#row_ping_port').css('display', 'none');
-			$('#row_ping_timeout').css('display', 'none');
-			$('#row_ping_retries').css('display', 'none');
+			$('#row_ping_method').hide();
+			$('#row_ping_port').hide();
+			$('#row_ping_timeout').hide();
+			$('#row_ping_retries').hide();
 
 			break;
 		case '2': // snmp
 		case '5': // snmp sysDesc
 		case '6': // snmp getNext
-			$('#row_ping_method').css('display', 'none');
-			$('#row_ping_port').css('display', 'none');
-			$('#row_ping_timeout').css('display', '');
-			$('#row_ping_retries').css('display', '');
+			$('#row_ping_method').hide();
+			$('#row_ping_port').hide();
+			$('#row_ping_timeout').show();
+			$('#row_ping_retries').show();
 
 			break;
 		default: // ping ok
 			switch(ping_method) {
 			case '1': // ping icmp
-				$('#row_ping_method').css('display', '');
-				$('#row_ping_port').css('display', 'none');
-				$('#row_ping_timeout').css('display', '');
-				$('#row_ping_retries').css('display', '');
+				$('#row_ping_method').show();
+				$('#row_ping_port').hide();
+				$('#row_ping_timeout').show();
+				$('#row_ping_retries').show();
 
 				break;
 			case '2': // ping udp
 			case '3': // ping tcp
-				$('#row_ping_method').css('display', '');
-				$('#row_ping_port').css('display', '');
-				$('#row_ping_timeout').css('display', '');
-				$('#row_ping_retries').css('display', '');
+				$('#row_ping_method').show();
+				$('#row_ping_port').show();
+				$('#row_ping_timeout').show();
+				$('#row_ping_retries').show();
 
 				break;
 			}
@@ -1041,8 +1089,8 @@ function device_javascript() {
 
 		if ($('#snmp_version').val() == '0') {
 			methods = [
-				{ value: '0', text: 'None' },
-				{ value: '3', text: 'Ping' }
+				{ value: '0', text: '<?php print __('None');?>' },
+				{ value: '3', text: '<?php print __('Ping');?>' }
 			];
 
 			if ($('#availability_method').val() != '3' && $('#availability_method').val() != '0') {
