@@ -22,10 +22,14 @@
  +-------------------------------------------------------------------------+
 */
 
-function upgrade_to_1_1_8() {
-	// update graph_watermark if set
-	$current_graph_watermark = db_fetch_cell("SELECT `value` FROM `settings` WHERE name = 'graph_watermark'");
-	if ($current_graph_watermark == 'Copyright (C) 2004-2017 The Cacti Group') {
-		db_install_execute("DELETE FROM `settings` WHERE name = 'graph_watermark'");
-	}
+function upgrade_to_1_1_35() {
+	db_install_execute("DELETE FROM data_input_data 
+		WHERE data_input_field_id=0");
+
+	db_install_execute("DELETE FROM snmp_query_graph_rrd 
+		WHERE data_template_id=0 OR data_template_rrd_id=0");
+
+	db_install_execute("DELETE FROM snmp_query_graph_rrd_sv 
+		WHERE data_template_id=0");
 }
+
