@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2017 The Cacti Group                                 |
+ | Copyright (C) 2004-2018 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,7 +13,7 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | Cacti: The Complete RRDTool-based Graphing Solution                     |
+ | Cacti: The Complete RRDtool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
  | This code is designed, written, and maintained by the Cacti Group. See  |
  | about.php and/or the AUTHORS file for specific developer information.   |
@@ -40,6 +40,8 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 		return $database_sessions["$device:$port:$db_name"];
 	}
 
+	$odevice = $device;
+
 	$flags = array();
 	if ($db_type == 'mysql') {
 		// Using 'localhost' will force unix sockets mode, which breaks when attempting to use mysql on a different port
@@ -67,14 +69,29 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 			}
 			$cnn_id->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
 
-			// MySQL 5.7 forces NO_ZERO_DATE on
-			$cnn_id->query("SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'STRICT_TRANS_TABLES', ''))");
-			$cnn_id->query("SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'TRADITIONAL', ''))");
-			$cnn_id->query("SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'NO_ZERO_DATE', ''))");
-			$cnn_id->query("SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'NO_ZERO_IN_DATE', ''))");
-			$cnn_id->query("SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY', ''))");
+			$bad_modes = array(
+				'STRICT_TRANS_TABLES', 
+				'STRICT_ALL_TABLES', 
+				'TRADITIONAL', 
+				'NO_ZERO_DATE', 
+				'NO_ZERO_IN_DATE', 
+				'ONLY_FULL_GROUP_BY', 
+				'NO_AUTO_VALUE_ON_ZERO'
+			);
 
-			$database_sessions["$device:$port:$db_name"] = $cnn_id;
+			$database_sessions["$odevice:$port:$db_name"] = $cnn_id;
+
+			// Get rid of bad modes
+			$modes = explode(',', db_fetch_cell('SELECT @@sql_mode'));
+
+			foreach($modes as $mode) {
+				if (array_search($mode, $bad_modes) === false) {
+					$new_modes[] = $mode;
+				}
+			}
+			$sql_mode = implode(',', $new_modes);
+
+			db_execute('SET SESSION sql_mode = "' . $sql_mode . '"');
 
 			return $cnn_id;
 		} catch (PDOException $e) {
@@ -87,7 +104,7 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 		usleep(40000);
 	}
 
-	return FALSE;
+	return false;
 }
 
 /* db_close - closes the open connection
@@ -164,7 +181,7 @@ function db_execute_prepared($sql, $parms = array(), $log = true, $db_conn = fal
 
 			return true;
 		} elseif ($log) {
-			if ($en == 1213 || $en == 1205) { 
+			if ($en == 1213 || $en == 1205) {
 				$errors++;
 				if ($errors > 30) {
 					cacti_log("ERROR: Too many Lock/Deadlock errors occurred! SQL:'" . $sql . "'", true, 'DBCALL', POLLER_VERBOSITY_DEBUG);
@@ -186,7 +203,7 @@ function db_execute_prepared($sql, $parms = array(), $log = true, $db_conn = fal
 
 				return false;
 			} else {
-				cacti_log("ERROR: A DB Exec Failed!, Error:$en, SQL:'" . $sql . "'", FALSE, 'DBCALL');
+				cacti_log("ERROR: A DB Exec Failed!, Error:$en, SQL:'" . $sql . "'", false, 'DBCALL');
 				cacti_log('ERROR: A DB Exec Failed!, Error: ' . $errorinfo[2], false);
 				cacti_debug_backtrace('SQL');
 
@@ -212,7 +229,7 @@ function db_execute_prepared($sql, $parms = array(), $log = true, $db_conn = fal
    @param $col_name - use this column name instead of the first one
    @param $log - whether to log error messages, defaults to true
    @returns - (bool) the output of the sql query as a single variable */
-function db_fetch_cell($sql, $col_name = '', $log = TRUE, $db_conn = FALSE) {
+function db_fetch_cell($sql, $col_name = '', $log = true, $db_conn = false) {
 	return db_fetch_cell_prepared($sql, array(), $col_name, $log, $db_conn);
 }
 
@@ -565,7 +582,7 @@ function db_cacti_initialized($is_web = true) {
 	$db_conn = $database_sessions["$database_hostname:$database_port:$database_default"];
 
 	if (!is_object($db_conn)) {
-		return FALSE;
+		return false;
 	}
 
 	$query = $db_conn->prepare('SELECT cacti FROM version');
@@ -652,7 +669,7 @@ function db_update_table($table, $data, $removecolumns = false, $log = true, $db
 			// Check that column is correct and fix it
 			// FIXME: Need to still check default value
 			$arr = db_fetch_row("SHOW columns FROM `$table` LIKE '" . $column['name'] . "'", $log, $db_conn);
-			if ($column['type'] != $arr['Type'] || (isset($column['NULL']) && ($column['NULL'] ? 'YES' : 'NO') != $arr['Null']) 
+			if ($column['type'] != $arr['Type'] || (isset($column['NULL']) && ($column['NULL'] ? 'YES' : 'NO') != $arr['Null'])
 							    || (isset($column['auto_increment']) && ($column['auto_increment'] ? 'auto_increment' : '') != $arr['Extra'])) {
 				$sql = 'ALTER TABLE `' . $table . '` CHANGE `' . $column['name'] . '` `' . $column['name'] . '`';
 				if (isset($column['type']))
@@ -942,7 +959,7 @@ function _db_replace($db_conn, $table, $fieldArray, $keyCols, $has_autoinc) {
    @param $table_name - the name of the table to make the replacement in
    @param $key_cols - the primary key(s)
    @returns - the auto incriment id column (if applicable) */
-function sql_save($array_items, $table_name, $key_cols = 'id', $autoinc = TRUE, $db_conn = false) {
+function sql_save($array_items, $table_name, $key_cols = 'id', $autoinc = true, $db_conn = false) {
 	global $database_sessions, $database_default, $database_hostname, $database_port;
 
 	/* check for a connection being passed, if not use legacy behavior */
@@ -955,9 +972,9 @@ function sql_save($array_items, $table_name, $key_cols = 'id', $autoinc = TRUE,
 	cacti_log("DEVEL: SQL Save on table '$table_name': '" . serialize($array_items) . "'", false, 'DBCALL', POLLER_VERBOSITY_DEVDBG);
 
 	foreach ($array_items as $key => $value) {
-		if (strstr($cols[$key]['type'], 'int') !== false || 
-			strstr($cols[$key]['type'], 'float') !== false || 
-			strstr($cols[$key]['type'], 'double') !== false || 
+		if (strstr($cols[$key]['type'], 'int') !== false ||
+			strstr($cols[$key]['type'], 'float') !== false ||
+			strstr($cols[$key]['type'], 'double') !== false ||
 			strstr($cols[$key]['type'], 'decimal') !== false) {
 			if ($value == '') {
 				if ($cols[$key]['null'] == 'YES') {
