@@ -30,14 +30,14 @@ function get_rrdfile_names() {
 	return db_fetch_assoc('SELECT local_data_id, data_source_path FROM data_template_data WHERE local_data_id != 0');
 }
 
-/* dsstats_debug - this simple routine echo's a standard message to the console
+/* dsstats_debug - this simple routine print's a standard message to the console
      when running in debug mode.
    @returns - NULL */
 function dsstats_debug($message) {
 	global $debug;
 
 	if ($debug) {
-		echo 'DSSTATS: ' . $message . "\n";
+		print 'DSSTATS: ' . $message . "\n";
 	}
 }
 
@@ -52,8 +52,16 @@ function dsstats_get_and_store_ds_avgpeak_values($interval) {
 	$rrdfiles = get_rrdfile_names();
 	$stats    = array();
 
-	/* open a pipe to rrdtool for writing */
-	$rrdtool_pipe = rrd_init();
+	$use_proxy = (read_config_option('storage_location') != '' ? true:false);
+
+	/* open a pipe to rrdtool for writing and reading */
+	if ($use_proxy) {
+		$rrdtool_pipe = rrd_init(false);
+	}else {
+		$process_pipes = dsstats_rrdtool_init();
+		$process       = $process_pipes[0];
+		$rrdtool_pipe  = $process_pipes[1];
+	}
 
 	if (sizeof($rrdfiles)) {
 		foreach ($rrdfiles as $file) {
@@ -62,13 +70,22 @@ function dsstats_get_and_store_ds_avgpeak_values($interval) {
 
 				$stats[$file['local_data_id']] = dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, $rrdtool_pipe);
 			} else {
-				$data_source_name = db_fetch_cell_prepared('SELECT name_cache FROM data_template_data WHERE local_data_id = ?', array($file['local_data_id']));
+				$data_source_name = db_fetch_cell_prepared('SELECT name_cache 
+					FROM data_template_data 
+					WHERE local_data_id = ?', 
+					array($file['local_data_id']));
+
 				cacti_log("WARNING: Data Source '$data_source_name' is damaged and contains no path.  Please delete and re-create both the Graph and Data Source.", false, 'DSSTATS');
 			}
 		}
 	}
 
-	rrd_close($rrdtool_pipe);
+	if ($use_proxy) {
+		rrd_close($rrdtool_pipe);
+	} else {
+		dsstats_rrdtool_close($process);
+	}
+
 	dsstats_write_buffer($stats, $interval);
 }
 
@@ -91,35 +108,29 @@ function dsstats_write_buffer(&$stats_array, $interval) {
 
 	/* don't attempt to process an empty array */
 	if (sizeof($stats_array)) {
-	foreach($stats_array as $local_data_id => $stats) {
-		/* some additional sanity checking */
-		if (sizeof($stats)) {
-		foreach($stats as $rrd_name => $avgpeak_stats) {
-			if ($i == 1) {
-				$delim = ' ';
-			} else {
-				$delim = ', ';
-			}
+		foreach($stats_array as $local_data_id => $stats) {
+			/* some additional sanity checking */
+			if (is_array($stats) && sizeof($stats)) {
+				foreach($stats as $rrd_name => $avgpeak_stats) {
+					$outbuf .= ($i == 1 ? ' ':', ') . "('" . $local_data_id . "','" .
+						$rrd_name . "','" .
+						$avgpeak_stats['AVG'] . "','" .
+						$avgpeak_stats['MAX'] . "')";
 
-			$outbuf .= $delim . "('" . $local_data_id . "','" .
-				$rrd_name . "','" .
-				$avgpeak_stats['AVG'] . "','" .
-				$avgpeak_stats['MAX'] . "')";
+					$out_length += strlen($outbuf);
 
-			$out_length += strlen($outbuf);
+					if (($out_length + $overhead) > $max_packet) {
+						db_execute($sql_prefix . $outbuf . $sql_suffix);
 
-			if (($out_length + $overhead) > $max_packet) {
-				db_execute($sql_prefix . $outbuf . $sql_suffix);
-
-				$outbuf     = '';
-				$out_length = 0;
-				$i          = 1;
-			} else {
-				$i++;
+						$outbuf     = '';
+						$out_length = 0;
+						$i          = 1;
+					} else {
+						$i++;
+					}
+				}
 			}
 		}
-		}
-	}
 	}
 
 	/* flush the buffer if it still has elements in it */
@@ -145,7 +156,9 @@ function dsstats_write_buffer(&$stats_array, $interval) {
 function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, $rrdtool_pipe) {
 	global $config;
 
-	if (read_config_option('storage_location')) {
+	$use_proxy = (read_config_option('storage_location') != '' ? true:false);
+
+	if ($use_proxy) {
 		$file_exists = rrdtool_execute("file_exists $rrdfile", true, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, 'DSSTATS');
 	}else {
 		$file_exists = file_exists($rrdfile);
@@ -154,7 +167,11 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, $rrdtool
 	/* don't attempt to get information if the file does not exist */
 	if ($file_exists) {
 		/* high speed or snail speed */
-		$info = rrdtool_execute("info $rrdfile", false, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'DSSTATS');
+		if ($use_proxy) {
+			$info = rrdtool_execute("info $rrdfile", false, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'DSSTATS');
+		} else {
+			$info = dsstats_rrdtool_execute("info $rrdfile", $rrdtool_pipe);
+		}
 
 		/* don't do anything if RRDfile did not return data */
 		if ($info != '') {
@@ -242,7 +259,12 @@ function dsstats_obtain_data_source_avgpeak_values($rrdfile, $interval, $rrdtool
 
 			/* now execute the xport command */
 			$xport_cmd = 'xport --start now-1' . $interval . ' --end now ' . trim($def) . ' ' . trim($xport) . ' --maxrows 10';
-			$xport_data = rrdtool_execute($xport_cmd, false, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'DSSTATS');
+
+			if ($use_proxy) {
+				$xport_data = rrdtool_execute($xport_cmd, false, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'DSSTATS');
+			} else {
+				$xport_data = dsstats_rrdtool_execute($xport_cmd, $rrdtool_pipe);
+			}
 
 			/* initialize the array of return values */
 			foreach($dsnames as $dsname => $present) {
@@ -328,11 +350,12 @@ function log_dsstats_statistics($type) {
 	$end = microtime(true);
 
 	$cacti_stats = sprintf('Time:%01.4f ', round($end-$start,4));
+
 	/* take time and log performance data */
 	$start = microtime(true);
 
 	/* log to the database */
-	db_execute_prepared('REPLACE INTO settings (name, value) VALUES (?, ?)', array('stats_dsstats_' . $type, $cacti_stats));
+	set_config_option('stats_dsstats_' . $type, $cacti_stats);
 
 	/* log to the logfile */
 	cacti_log('DSSTATS STATS: Type:' . $type . ', ' . $cacti_stats , true, 'SYSTEM');
@@ -649,7 +672,7 @@ function dsstats_poller_output(&$rrd_update_array) {
 
 							$n++;
 
-							if (($n % 1000) == 0) echo '.';
+							if (($n % 1000) == 0) print '.';
 						}
 					}
 				}
@@ -680,9 +703,7 @@ function dsstats_boost_bottom() {
 		include_once($config['base_path'] . '/lib/rrd.php');
 
 		/* run the daily stats. log to database to prevent secondary runs */
-		db_execute("REPLACE INTO settings
-			(name, value) VALUES
-			('dsstats_last_daily_run_time', '" . date('Y-m-d G:i:s', time()) . "')");
+		set_config_option('dsstats_last_daily_run_time', date('Y-m-d G:i:s', time()));
 
 		dsstats_get_and_store_ds_avgpeak_values('daily');
 
@@ -729,3 +750,80 @@ function dsstats_poller_bottom () {
 		exec_background($command_string, $extra_args);
 	}
 }
+
+/* dsstats_rrdtool_init - this routine provides a bi-directional socket based connection to RRDtool.
+     it provides a high speed connection to rrdfile in the case where the traditional Cacti call does
+     not when performing fetch type calls.
+   @returns - (mixed) An array that includes both the process resource and the pipes to communicate
+     with RRDtool. */
+function dsstats_rrdtool_init() {
+	global $config;
+
+	if ($config['cacti_server_os'] == 'unix') {
+		$fds = array(
+			0 => array('pipe', 'r'), // stdin
+			1 => array('pipe', 'w'), // stdout
+			2 => array('file', '/dev/null', 'a')  // stderr
+		);
+	} else {
+		$fds = array(
+			0 => array('pipe', 'r'), // stdin
+			1 => array('pipe', 'w'), // stdout
+			2 => array('file', 'nul', 'a')  // stderr
+		);
+	}
+
+	/* set the rrdtool default font */
+	if (read_config_option('path_rrdtool_default_font')) {
+		putenv('RRD_DEFAULT_FONT=' . read_config_option('path_rrdtool_default_font'));
+	}
+
+	$command = read_config_option('path_rrdtool') . ' - ';
+
+	$process = proc_open($command, $fds, $pipes);
+
+	/* make stdin/stdout/stderr non-blocking */
+	stream_set_blocking($pipes[0], 0);
+	stream_set_blocking($pipes[1], 0);
+
+	return array($process, $pipes);
+}
+
+/* dsstats_rrdtool_execute - this routine passes commands to RRDtool and returns the information
+     back to DSStats.  It is important to note here that RRDtool needs to provide an either 'OK'
+     or 'ERROR' response accross the pipe as it does not provide EOF characters to key upon.
+     This may not be the best method and may be changed after I have a conversation with a few
+     developers.
+   @arg $command - (string) The rrdtool command to execute
+   @arg $pipes - (array) An array of stdin and stdout pipes to read and write data from
+   @returns - (string) The output from RRDtool */
+function dsstats_rrdtool_execute($command, $pipes) {
+	$stdout = '';
+
+	if ($command == '') return;
+
+	$command .= "\r\n";
+	$return_code = fwrite($pipes[0], $command);
+
+	while (!feof($pipes[1])) {
+		$stdout .= fgets($pipes[1], 4096);
+
+		if (substr_count($stdout, 'OK')) {
+			break;
+		}
+
+		if (substr_count($stdout, 'ERROR')) {
+			break;
+		}
+	}
+
+	if (strlen($stdout)) return $stdout;
+}
+
+/* dsstats_rrdtool_close - this routine closes the RRDtool process thus also
+     closing the pipes.
+   @returns - NULL */
+function dsstats_rrdtool_close($process) {
+	proc_close($process);
+}
+
