@@ -78,7 +78,7 @@ function html_start_box($title, $width, $div, $cell_padding, $align, $add_text,
 		print "<div class='cactiTableTitle'><span>" . ($title != '' ? $title:'') . '</span></div>';
 		print "<div class='cactiTableButton'>\n";
 		if ($add_text != '' && !is_array($add_text)) {
-			print "<span><a class='linkOverDark' title='$add_label' href='" . html_escape($add_text) . "'><i class='fa fa-plus'></i></a></span>";
+			print "<span class='cactiFilterAdd' title='$add_label'><a class='linkOverDark' href='" . html_escape($add_text) . "'><i class='fa fa-plus'></i></a></span>";
 		} else {
 			if (is_array($add_text)) {
 				if (sizeof($add_text)) {
@@ -107,7 +107,7 @@ function html_start_box($title, $width, $div, $cell_padding, $align, $add_text,
 							$title = $add_label;
 						}
 
-						print "<span><a" . (isset($icon['id']) ? "id='" . $icon['id'] . "'":"") . " class='$classo' href='$href' title='$title'><i class='$classi'></i></a></span>";
+						print "<span class='cactiFilterAdd' title='$title'><a" . (isset($icon['id']) ? "id='" . $icon['id'] . "'":"") . " class='$classo' href='$href'><i class='$classi'></i></a></span>";
 					}
 				}
 			} else {
@@ -1843,7 +1843,7 @@ function html_spikekill_setting($name) {
 function html_spikekill_menu($local_graph_id) {
 	$ravgnan  = '<li>' . __('Replacement Method') . '<ul>';
 	$ravgnan .= '<li class="skmethod" id="method_avg"><i ' . (html_spikekill_setting('spikekill_avgnan') == 'avg' ? 'class="fa fa-check"':'') . '></i><span></span>' . __('Average') . '</li>';
-	$ravgnan .= '<li class="skmethod" id="method_nan"><i ' . (html_spikekill_setting('spikekill_avgnan') == 'nan' ? 'class="fa fa-check"':'') . '></i><span></span>' . __('Nan\'s') . '</li>';
+	$ravgnan .= '<li class="skmethod" id="method_nan"><i ' . (html_spikekill_setting('spikekill_avgnan') == 'nan' ? 'class="fa fa-check"':'') . '></i><span></span>' . __('NaN\'s') . '</li>';
 	$ravgnan .= '<li class="skmethod" id="method_last"><i ' . (html_spikekill_setting('spikekill_avgnan') == 'last' ? 'class="fa fa-check"':'') . '></i><span></span>' . __('Last Known Good') . '</li>';
 	$ravgnan .= '</ul></li>';
 
@@ -1908,6 +1908,10 @@ function html_spikekill_js() {
 			}
 		});
 
+		$('span.spikekill').children().contextmenu(function() {
+			return false;
+		});
+
 		$('span.spikekill').unbind().click(function() {
 			if (spikeKillOpen == false) {
 				local_graph_id = $(this).attr('data-graph');
@@ -2045,10 +2049,12 @@ function html_spikekill_js() {
 	<?php
 }
 
-function html_common_header($title) {
+function html_common_header($title, $selectedTheme = '') {
 	global $config;
 
-	$selectedTheme = get_selected_theme();
+	if ($selectedTheme == '') {
+		$selectedTheme = get_selected_theme();
+	}
 
 	?>
 	<meta http-equiv='X-UA-Compatible' content='IE=Edge,chrome=1'>
@@ -2062,13 +2068,13 @@ function html_common_header($title) {
 	<link href='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/images/favicon.ico' rel='shortcut icon'>
 	<link href='<?php echo $config['url_path']; ?>include/themes/<?php print $selectedTheme;?>/images/cacti_logo.gif' rel='icon' sizes='96x96'>
 	<?php
-	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery.zoom.css'); 
-	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery-ui.css'); 
+	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery.zoom.css');
+	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery-ui.css');
 	print get_md5_include_css('include/themes/' . $selectedTheme .'/default/style.css');
 	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery.multiselect.css');
 	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery.timepicker.css');
 	print get_md5_include_css('include/themes/' . $selectedTheme .'/jquery.colorpicker.css');
-	print get_md5_include_css('include/themes/' . $selectedTheme .'/c3.css'); 
+	print get_md5_include_css('include/themes/' . $selectedTheme .'/c3.css');
 	print get_md5_include_css('include/themes/' . $selectedTheme .'/pace.css');
 	print get_md5_include_css('include/fa/css/font-awesome.css');
 	print get_md5_include_css('include/themes/' . $selectedTheme .'/main.css');
